@@ -29,12 +29,14 @@ DEFAULT_OPTIONS = {
     'table': {
         'reads_lower_threshold': 30,
         'reads_upper_threshold': 90,
+        'throttled_reads_upper_threshold': 0,
         'increase_reads_with': 50,
         'decrease_reads_with': 50,
         'increase_reads_unit': 'percent',
         'decrease_reads_unit': 'percent',
         'writes_lower_threshold': 30,
         'writes_upper_threshold': 90,
+        'throttled_writes_upper_threshold': 0,
         'increase_writes_with': 50,
         'decrease_writes_with': 50,
         'increase_writes_unit': 'percent',
@@ -47,16 +49,20 @@ DEFAULT_OPTIONS = {
         'allow_scaling_down_writes_on_0_percent': False,
         'always_decrease_rw_together': False,
         'maintenance_windows': None,
+        'sns_topic_arn': None,
+        'sns_message_types': []
     },
     'gsi': {
         'reads_lower_threshold': 30,
         'reads_upper_threshold': 90,
+        'throttled_reads_upper_threshold': 0,
         'increase_reads_with': 50,
         'decrease_reads_with': 50,
         'increase_reads_unit': 'percent',
         'decrease_reads_unit': 'percent',
         'writes_lower_threshold': 30,
         'writes_upper_threshold': 90,
+        'throttled_writes_upper_threshold': 0,
         'increase_writes_with': 50,
         'decrease_writes_with': 50,
         'increase_writes_unit': 'percent',
@@ -69,6 +75,8 @@ DEFAULT_OPTIONS = {
         'allow_scaling_down_writes_on_0_percent': False,
         'always_decrease_rw_together': False,
         'maintenance_windows': None,
+        'sns_topic_arn': None,
+        'sns_message_types': []
     }
 }
 
@@ -86,11 +94,10 @@ def get_configuration():
     cmd_line_options = command_line_parser.parse()
 
     # If a configuration file is specified, read that as well
+    conf_file_options = None
     if 'config' in cmd_line_options:
         conf_file_options = config_file_parser.parse(
             cmd_line_options['config'])
-    else:
-        conf_file_options = None
 
     # Extract global config
     configuration['global'] = __get_global_options(
@@ -156,7 +163,20 @@ def __get_config_table_options(conf_file_options):
         for option in DEFAULT_OPTIONS['table'].keys():
             options[table_name][option] = DEFAULT_OPTIONS['table'][option]
 
-            if option in conf_file_options['tables'][table_name]:
+            if option not in conf_file_options['tables'][table_name]:
+                continue
+
+            if option == 'sns_message_types':
+                try:
+                    raw_list = conf_file_options['tables'][table_name][option]
+                    options[table_name][option] = \
+                        [i.strip() for i in raw_list.split(',')]
+                except:
+                    print(
+                        'Error parsing the "sns-message-types" '
+                        'option: {0}'.format(
+                            conf_file_options['tables'][table_name][option]))
+            else:
                 options[table_name][option] = \
                     conf_file_options['tables'][table_name][option]
 
@@ -164,12 +184,27 @@ def __get_config_table_options(conf_file_options):
         if 'gsis' in conf_file_options['tables'][table_name]:
             for gsi_name in conf_file_options['tables'][table_name]['gsis']:
                 for option in DEFAULT_OPTIONS['gsi'].keys():
-                    if (option in conf_file_options[
+                    opt = DEFAULT_OPTIONS['gsi'][option]
+
+                    if (option not in conf_file_options[
                             'tables'][table_name]['gsis'][gsi_name]):
+                        continue
+
+                    if option == 'sns_message_types':
+                        try:
+                            raw_list = conf_file_options[
+                                'tables'][table_name]['gsis'][gsi_name][option]
+                            opt = [i.strip() for i in raw_list.split(',')]
+                        except:
+                            print(
+                                'Error parsing the "sns-message-types" '
+                                'option: {0}'.format(
+                                    conf_file_options[
+                                        'tables'][table_name][
+                                            'gsis'][gsi_name][option]))
+                    else:
                         opt = conf_file_options[
                             'tables'][table_name]['gsis'][gsi_name][option]
-                    else:
-                        opt = DEFAULT_OPTIONS['gsi'][option]
 
                     if 'gsis' not in options[table_name]:
                         options[table_name]['gsis'] = {}
@@ -229,7 +264,7 @@ def __get_logging_options(cmd_line_options, conf_file_options=None):
 
 
 def __check_gsi_rules(configuration):
-    """ Do some basic checks on the configuraion """
+    """ Do some basic checks on the configuration """
     for table_name in configuration['tables']:
         if not 'gsis' in configuration['tables'][table_name]:
             continue
@@ -278,6 +313,15 @@ def __check_gsi_rules(configuration):
                     'Setting --increase-reads-with to 100.')
                 gsi['increase_reads_with'] = 100
 
+            # Check sns-message-types
+            valid_sns_message_types = ['scale-up', 'scale-down']
+            if gsi['sns_message_types']:
+                for sns_type in gsi['sns_message_types']:
+                    if sns_type not in valid_sns_message_types:
+                        print('Warning: Invalid sns-message-type: {0}'.format(
+                            sns_type))
+                        gsi['sns_message_types'].remove(sns_type)
+
             # Ensure values > 1 for some important configuration options
             options = [
                 'reads_lower_threshold',
@@ -334,7 +378,7 @@ def __check_logging_rules(configuration):
 
 
 def __check_table_rules(configuration):
-    """ Do some basic checks on the configuraion """
+    """ Do some basic checks on the configuration """
     for table_name in configuration['tables']:
         table = configuration['tables'][table_name]
         # Check that increase/decrease units is OK
@@ -354,6 +398,15 @@ def __check_table_rules(configuration):
                 'decrease-writes-with must be set to either percent or units')
             sys.exit(1)
 
+        # Check sns-message-types
+        valid_sns_message_types = ['scale-up', 'scale-down']
+        if table['sns_message_types']:
+            for sns_type in table['sns_message_types']:
+                if sns_type not in valid_sns_message_types:
+                    print('Warning: Invalid sns-message-type: {0}'.format(
+                        sns_type))
+                    table['sns_message_types'].remove(sns_type)
+
         # Check that increase_writes_with is not > 100
         if (table['increase_writes_unit'] == 'percent' and
                 table['increase_writes_with'] > 100):
